@@ -16,7 +16,7 @@ pub const CONTROL_ALT_SHIFT: KeyModifiers = KeyModifiers::from_bits_truncate(
     KeyModifiers::ALT.bits() | KeyModifiers::CONTROL.bits() | KeyModifiers::SHIFT.bits(),
 );
 
-#[derive(Clone, Deserialize, Serialize)]
+#[derive(Clone, Deserialize, Serialize, Debug)]
 pub struct Keys {
     pub global_esc: BindingForEvent,
     pub global_quit: BindingForEvent,
@@ -184,7 +184,7 @@ impl Keys {
     }
 }
 
-#[derive(Clone, Deserialize, Copy, Eq, PartialEq, Hash, Serialize)]
+#[derive(Clone, Deserialize, Copy, Eq, PartialEq, Hash, Serialize, Debug)]
 pub struct BindingForEvent {
     pub code: Key,
     pub modifier: KeyModifiers,
