@@ -1,4 +1,4 @@
-use anyhow::{Context, Result};
+use anyhow::{anyhow, Context, Result};
 use std::path::{Path, PathBuf};
 
 use crate::track::Track;
@@ -33,7 +33,9 @@ pub struct Database {
 
 impl Database {
     /// Creates a new connection to the database (and creates database if
-    /// it does not already exist). Panics if database cannot be accessed.
+    /// it does not already exist).
+    /// # Panics
+    /// if database cannot be accessed.
     pub fn connect(path: &Path) -> Result<Database> {
         let mut db_path = path.to_path_buf();
         std::fs::create_dir_all(&db_path)
@@ -50,11 +52,11 @@ impl Database {
             let conn = db_conn
                 .conn
                 .as_ref()
-                .expect("Error connecting to database.");
+                .ok_or(anyhow!("Error connecting to database."))?;
 
             // SQLite defaults to foreign key support off
             conn.execute("PRAGMA foreign_keys=ON;", params![])
-                .expect("Could not set database parameters.");
+                .with_context(|| "Could not set database parameters.")?;
 
             // get version number stored in database
             let mut stmt = conn.prepare("SELECT version FROM version WHERE id = 1;")?;
@@ -90,8 +92,12 @@ impl Database {
     /// Creates the necessary database tables, if they do not already
     /// exist. Panics if database cannot be accessed, or if tables cannot
     /// be created.
+    #[allow(clippy::missing_panics_doc)]
     pub fn create(&self) -> Result<()> {
-        let conn = self.conn.as_ref().expect("Error connecting to database.");
+        let conn = self
+            .conn
+            .as_ref()
+            .ok_or(anyhow!("Error connecting to database."))?;
 
         // create podcasts table
         conn.execute(
@@ -157,7 +163,10 @@ impl Database {
     /// of the app, this updates the value stored in the database to
     /// match.
     fn update_version(&self, current_version: &Version, update: bool) -> Result<()> {
-        let conn = self.conn.as_ref().expect("Error connecting to database.");
+        let conn = self
+            .conn
+            .as_ref()
+            .ok_or(anyhow!("Error connecting to database."))?;
 
         if update {
             conn.execute(
@@ -177,8 +186,10 @@ impl Database {
 
     /// Inserts a new podcast and list of podcast episodes into the
     /// database.
+    #[allow(clippy::missing_panics_doc)]
     pub fn insert_podcast(&self, podcast: &PodcastNoId) -> Result<SyncResult> {
-        let mut conn = Connection::open(&self.path).expect("Error connecting to database.");
+        let mut conn =
+            Connection::open(&self.path).with_context(|| "Error connecting to database.")?;
         let tx = conn.transaction()?;
         // let conn = self.conn.as_ref().expect("Error connecting to database.");
         {
@@ -253,8 +264,12 @@ impl Database {
     }
 
     /// Inserts a filepath to a downloaded episode.
+    #[allow(clippy::missing_panics_doc)]
     pub fn insert_file(&self, episode_id: i64, path: &Path) -> Result<()> {
-        let conn = self.conn.as_ref().expect("Error connecting to database.");
+        let conn = self
+            .conn
+            .as_ref()
+            .ok_or(anyhow!("Error connecting to database."))?;
 
         let mut stmt = conn.prepare_cached(
             "INSERT INTO files (episode_id, path)
@@ -266,16 +281,24 @@ impl Database {
 
     /// Removes a file listing for an episode from the database when the
     /// user has chosen to delete the file.
+    #[allow(clippy::missing_panics_doc)]
     pub fn remove_file(&self, episode_id: i64) -> Result<()> {
-        let conn = self.conn.as_ref().expect("Error connecting to database.");
+        let conn = self
+            .conn
+            .as_ref()
+            .ok_or(anyhow!("Error connecting to database."))?;
         let mut stmt = conn.prepare_cached("DELETE FROM files WHERE episode_id = ?;")?;
         stmt.execute(params![episode_id])?;
         Ok(())
     }
 
     /// Removes all file listings for the selected episode ids.
+    #[allow(clippy::missing_panics_doc)]
     pub fn remove_files(&self, episode_ids: &[i64]) -> Result<()> {
-        let conn = self.conn.as_ref().expect("Error connecting to database.");
+        let conn = self
+            .conn
+            .as_ref()
+            .ok_or(anyhow!("Error connecting to database."))?;
 
         // convert list of episode ids into a comma-separated String
         let episode_list: Vec<String> = episode_ids
@@ -290,8 +313,12 @@ impl Database {
     }
 
     /// Removes a podcast, all episodes, and files from the database.
+    #[allow(clippy::missing_panics_doc)]
     pub fn remove_podcast(&self, podcast_id: i64) -> Result<()> {
-        let conn = self.conn.as_ref().expect("Error connecting to database.");
+        let conn = self
+            .conn
+            .as_ref()
+            .ok_or(anyhow!("Error connecting to database."))?;
         // Note: Because of the foreign key constraints on `episodes`
         // and `files` tables, all associated episodes for this podcast
         // will also be deleted, and all associated file entries for
@@ -304,9 +331,13 @@ impl Database {
     /// Updates an existing podcast in the database, where metadata is
     /// changed if necessary, and episodes are updated (modified episodes
     /// are updated, new episodes are inserted).
+    #[allow(clippy::missing_panics_doc)]
     pub fn update_podcast(&self, pod_id: i64, podcast: &PodcastNoId) -> Result<SyncResult> {
         {
-            let conn = self.conn.as_ref().expect("Error connecting to database.");
+            let conn = self
+                .conn
+                .as_ref()
+                .ok_or(anyhow!("Error connecting to database."))?;
             let mut stmt = conn.prepare_cached(
                 "UPDATE podcasts SET title = ?, url = ?, description = ?,
             author = ?, explicit = ?, last_checked = ?
@@ -349,7 +380,8 @@ impl Database {
             }
         }
 
-        let mut conn = Connection::open(&self.path).expect("Error connecting to database.");
+        let mut conn =
+            Connection::open(&self.path).with_context(|| "Error connecting to database.")?;
         let tx = conn.transaction()?;
 
         let mut insert_ep = Vec::new();
@@ -454,8 +486,12 @@ impl Database {
     }
 
     /// Updates an episode to mark it as played or unplayed.
+    #[allow(clippy::missing_panics_doc)]
     pub fn set_played_status(&self, episode_id: i64, played: bool) -> Result<()> {
-        let conn = self.conn.as_ref().expect("Error connecting to database.");
+        let conn = self
+            .conn
+            .as_ref()
+            .ok_or(anyhow!("Error connecting to database."))?;
 
         let mut stmt = conn.prepare_cached("UPDATE episodes SET played = ? WHERE id = ?;")?;
         stmt.execute(params![played, episode_id])?;
@@ -463,8 +499,10 @@ impl Database {
     }
 
     /// Updates an episode to mark it as played or unplayed.
+    #[allow(clippy::missing_panics_doc)]
     pub fn set_all_played_status(&self, episode_id_vec: &[i64], played: bool) -> Result<()> {
-        let mut conn = Connection::open(&self.path).expect("Error connecting to database.");
+        let mut conn =
+            Connection::open(&self.path).with_context(|| "Error connecting to database.")?;
         let tx = conn.transaction()?;
 
         for episode_id in episode_id_vec {
@@ -478,8 +516,12 @@ impl Database {
     /// Updates an episode to "remove" it by hiding it. "Removed"
     /// episodes need to stay in the database so that they don't get
     /// re-added when the podcast is synced again.
+    #[allow(clippy::missing_panics_doc)]
     pub fn hide_episode(&self, episode_id: i64, hide: bool) -> Result<()> {
-        let conn = self.conn.as_ref().expect("Error connecting to database.");
+        let conn = self
+            .conn
+            .as_ref()
+            .ok_or(anyhow!("Error connecting to database."))?;
 
         let mut stmt = conn.prepare_cached("UPDATE episodes SET hidden = ? WHERE id = ?;")?;
         stmt.execute(params![hide, episode_id])?;
@@ -488,8 +530,12 @@ impl Database {
 
     /// Generates list of all podcasts in database.
     /// TODO: This should probably use a JOIN statement instead.
+    #[allow(clippy::missing_panics_doc)]
     pub fn get_podcasts(&self) -> Result<Vec<Podcast>> {
-        let conn = self.conn.as_ref().expect("Error connecting to database.");
+        let conn = self
+            .conn
+            .as_ref()
+            .ok_or(anyhow!("Error connecting to database."))?;
         let mut stmt = conn.prepare_cached("SELECT * FROM podcasts;")?;
         let podcast_iter = stmt.query_map(params![], |row| {
             let pod_id = row.get("id")?;
@@ -530,8 +576,12 @@ impl Database {
     }
 
     /// Generates list of episodes for a given podcast.
+    #[allow(clippy::missing_panics_doc)]
     pub fn get_episodes(&self, pod_id: i64, include_hidden: bool) -> Result<Vec<Episode>> {
-        let conn = self.conn.as_ref().expect("Error connecting to database.");
+        let conn = self
+            .conn
+            .as_ref()
+            .ok_or(anyhow!("Error connecting to database."))?;
         let mut stmt = if include_hidden {
             conn.prepare_cached(
                 "SELECT * FROM episodes
@@ -573,14 +623,19 @@ impl Database {
     }
 
     /// Deletes all rows in all tables
+    #[allow(clippy::missing_panics_doc)]
     pub fn clear_db(&self) -> Result<()> {
-        let conn = self.conn.as_ref().expect("Error connecting to database.");
+        let conn = self
+            .conn
+            .as_ref()
+            .ok_or(anyhow!("Error connecting to database."))?;
         conn.execute("DELETE FROM files;", params![])?;
         conn.execute("DELETE FROM episodes;", params![])?;
         conn.execute("DELETE FROM podcasts;", params![])?;
         Ok(())
     }
 
+    #[allow(clippy::missing_panics_doc)]
     pub fn get_last_position(&mut self, track: &Track) -> Result<Duration> {
         let query = "SELECT last_position FROM episodes WHERE url = ?1";
 
@@ -588,22 +643,25 @@ impl Database {
         let conn = self
             .conn
             .as_ref()
-            .expect("conn is not available for get last position.");
+            .ok_or(anyhow!("conn is not available for get last position."))?;
         conn.query_row(
             query,
             params![track.file().unwrap_or("Unknown File").to_string(),],
             |row| {
                 let last_position_u64: u64 = row.get(0)?;
-                // eprintln!("last_position_u64 is {last_position_u64}");
+                // error!("last_position_u64 is {last_position_u64}");
                 last_position = Duration::from_secs(last_position_u64);
                 Ok(last_position)
             },
         )?;
         // .expect("get last position failed.");
-        // eprintln!("get last pos as {}", last_position.as_secs());
+        // error!("get last pos as {}", last_position.as_secs());
         Ok(last_position)
     }
 
+    /// # Panics
+    ///
+    /// if the connection is unavailable
     pub fn set_last_position(&mut self, track: &Track, last_position: Duration) {
         let query = "UPDATE episodes SET last_position = ?1 WHERE url = ?2";
         let conn = self
@@ -618,7 +676,7 @@ impl Database {
             ],
         )
         .expect("update last position failed.");
-        // eprintln!("set last position as {}", last_position.as_secs());
+        // error!("set last position as {}", last_position.as_secs());
     }
 }
 
@@ -626,9 +684,7 @@ impl Database {
 /// `DateTime`<Utc> object
 fn convert_date(result: &Result<i64, rusqlite::Error>) -> Option<DateTime<Utc>> {
     match result {
-        Ok(timestamp) => {
-            NaiveDateTime::from_timestamp_opt(*timestamp, 0).map(|ndt| DateTime::from_utc(ndt, Utc))
-        }
+        Ok(timestamp) => DateTime::from_timestamp(*timestamp, 0),
         Err(_) => None,
     }
 }
