@@ -288,6 +288,10 @@ impl PodcastFeed {
 }
 
 /// Spawns a new thread to check a feed and retrieve podcast data.
+///
+/// # Panics
+///
+/// if sending commands to `tx_to_main` fails
 pub fn check_feed(
     feed: PodcastFeed,
     max_retries: usize,
@@ -420,20 +424,10 @@ fn parse_episode_data(item: &Item) -> EpisodeNoId {
         Some(dsc) => dsc.to_string(),
         None => String::new(),
     };
-    let pubdate = match item.pub_date() {
-        Some(pd) => match parse_from_rfc2822_with_fallback(pd) {
-            Ok(date) => {
-                // this is a bit ridiculous, but it seems like
-                // you have to convert from a DateTime<FixedOffset>
-                // to a NaiveDateTime, and then from there create
-                // a DateTime<Utc>; see
-                // https://github.com/chronotope/chrono/issues/169#issue-239433186
-                Some(DateTime::from_utc(date.naive_utc(), Utc))
-            }
-            Err(_) => None,
-        },
-        None => None,
-    };
+    let pubdate = item
+        .pub_date()
+        .and_then(|pd| parse_from_rfc2822_with_fallback(pd).ok())
+        .map(std::convert::Into::into);
 
     let mut duration = None;
     let mut image_url = None;
@@ -544,6 +538,10 @@ impl Threadpool {
 
     /// Adds a new job to the threadpool, passing closure to first
     /// available worker.
+    ///
+    /// # Panics
+    ///
+    /// if sending commands to the sender fails
     pub fn execute<F>(&self, func: F)
     where
         F: FnOnce() + Send + 'static,
@@ -591,6 +589,7 @@ struct Worker {
 impl Worker {
     /// Creates a new Worker, which waits for Jobs to be passed by the
     /// Threadpool.
+    #[allow(clippy::missing_panics_doc)]
     fn new(receiver: Arc<Mutex<mpsc::Receiver<JobMessage>>>) -> Worker {
         let thread = thread::spawn(move || loop {
             let message = receiver
@@ -688,7 +687,7 @@ pub fn import_from_opml(db_path: &Path, config: &Settings, filepath: &str) -> Re
                     }
                     Err(_err) => {
                         failure = true;
-                        eprintln!("Error adding {title}");
+                        error!("Error adding {title}");
                     }
                 }
             }
@@ -697,9 +696,9 @@ pub fn import_from_opml(db_path: &Path, config: &Settings, filepath: &str) -> Re
                 msg_counter += 1;
                 failure = true;
                 if let Some(t) = feed.title {
-                    eprintln!("Error retrieving RSS feed: {t}");
+                    error!("Error retrieving RSS feed: {t}");
                 } else {
-                    eprintln!("Error retrieving RSS feed");
+                    error!("Error retrieving RSS feed");
                 }
             }
 
@@ -816,6 +815,7 @@ pub struct EpData {
 /// files to download. It uses the threadpool to start jobs
 /// for every episode to be downloaded. New jobs can be requested
 /// by the user while there are still ongoing jobs.
+#[allow(clippy::missing_panics_doc)]
 pub fn download_list(
     episodes: Vec<EpData>,
     dest: &Path,
@@ -827,16 +827,23 @@ pub fn download_list(
     for ep in episodes {
         let tx = tx_to_main.clone();
         let dest2 = dest.to_path_buf();
-        threadpool.execute(move || {
-            tx.send(Msg::Podcast(PCMsg::DLStart(ep.clone())))
-                .expect("Thread messaging error when start download");
-            let result = download_file(ep, dest2, max_retries);
-            tx.send(Msg::Podcast(result))
-                .expect("Thread messaging error");
-        });
+        threadpool.execute(move || download_job(&tx, ep, dest2, max_retries));
     }
 }
 
+/// # Panics
+///
+/// if sending command via `tx_to_main` fails
+fn download_job(tx_to_main: &Sender<Msg>, ep: EpData, dest: PathBuf, max_retries: usize) {
+    tx_to_main
+        .send(Msg::Podcast(PCMsg::DLStart(ep.clone())))
+        .expect("Thread messaging error when start download");
+    let result = download_file(ep, dest, max_retries);
+    tx_to_main
+        .send(Msg::Podcast(result))
+        .expect("Thread messaging error");
+}
+
 /// Downloads a file to a local filepath, returning `DownloadMsg` variant
 /// indicating success or failure.
 #[allow(clippy::single_match_else)]
@@ -913,7 +920,7 @@ fn download_file(mut ep_data: EpData, destination_path: PathBuf, mut max_retries
 
     match dst.unwrap().write_all(&resp_data) {
         // match std::io::copy(&mut resp_data, &mut dst.unwrap()) {
-        Ok(_) => PCMsg::DLComplete(ep_data),
+        Ok(()) => PCMsg::DLComplete(ep_data),
         Err(_) => PCMsg::DLFileWriteError(ep_data),
     }
 }
