@@ -24,7 +24,7 @@ use crate::podcast::Episode;
  */
 use crate::songtag::lrc::Lyric;
 use crate::utils::get_parent_folder;
-use anyhow::{bail, Result};
+use anyhow::{bail, Context, Result};
 use id3::frame::Lyrics;
 use lofty::id3::v2::{Frame, FrameFlags, FrameValue, Id3v2Tag, UnsynchronizedTextFrame};
 use lofty::{
@@ -40,6 +40,7 @@ use std::path::Path;
 use std::str::FromStr;
 use std::time::{Duration, SystemTime};
 
+// TODO: add some kind of identifier for easy printing, like a uri that is NOT optional
 #[derive(Clone, Debug)]
 pub struct Track {
     /// Artist of the song
@@ -49,6 +50,7 @@ pub struct Track {
     /// Title of the song
     title: Option<String>,
     /// File path to the song
+    // TODO: why is this a http(s) url in Radio case?
     file: Option<String>,
     /// Duration of the song
     duration: Duration,
@@ -56,6 +58,7 @@ pub struct Track {
     name: Option<String>,
     /// Extension of the song
     ext: Option<String>,
+    // TODO: why is this set to the http(s) url in Radio case?
     directory: Option<String>,
     pub last_modified: SystemTime,
     /// USLT lyrics
@@ -72,7 +75,7 @@ pub struct Track {
     // Performer
     // Disc
     // Comment
-    pub media_type: Option<MediaType>,
+    pub media_type: MediaType,
     pub podcast_localfile: Option<String>,
 }
 
@@ -118,7 +121,7 @@ impl Track {
             album_photo: ep.image_url.clone(),
             file_type: None,
             genre: None,
-            media_type: Some(MediaType::Podcast),
+            media_type: MediaType::Podcast,
             podcast_localfile,
         }
     }
@@ -146,7 +149,7 @@ impl Track {
                 song.album = tag.album().map(std::borrow::Cow::into_owned);
                 song.title = tag.title().map(std::borrow::Cow::into_owned);
                 song.genre = tag.genre().map(std::borrow::Cow::into_owned);
-                song.media_type = Some(MediaType::Music);
+                song.media_type = MediaType::Music;
 
                 if for_db {
                     return Ok(song);
@@ -220,7 +223,7 @@ impl Track {
         track.artist = Some("Radio".to_string());
         track.title = Some("Radio Station".to_string());
         track.album = Some("Live".to_string());
-        track.media_type = Some(MediaType::LiveRadio);
+        track.media_type = MediaType::LiveRadio;
         track
     }
     fn new<P: AsRef<Path>>(path: P) -> Self {
@@ -264,12 +267,12 @@ impl Track {
             album_photo,
             last_modified,
             genre,
-            media_type: Some(MediaType::Music),
+            media_type: MediaType::Music,
             podcast_localfile: None,
         }
     }
 
-    pub fn adjust_lyric_delay(&mut self, time_pos: i64, offset: i64) -> Result<()> {
+    pub fn adjust_lyric_delay(&mut self, time_pos: Duration, offset: i64) -> Result<()> {
         if let Some(lyric) = self.parsed_lyric.as_mut() {
             lyric.adjust_offset(time_pos, offset);
             let text = lyric.as_lrc_text();
@@ -445,7 +448,7 @@ impl Track {
                                         encoding: TextEncoding::UTF8,
                                         language: l.lang.as_bytes()[0..3]
                                             .try_into()
-                                            .expect("wrong length of language"),
+                                            .with_context(|| "wrong length of language")?,
                                         description: l.description,
                                         content: l.text,
                                     }),
