@@ -38,6 +38,7 @@ mod progress;
     clippy::doc_markdown,
     clippy::module_name_repetitions
 )]
+/// Tag Editor Controls
 mod tag_editor;
 mod xywh;
 mod youtube_search;
@@ -52,18 +53,15 @@ pub use music_library::MusicLibrary;
 pub use playlist::Playlist;
 pub use podcast::{EpisodeList, FeedsList};
 pub use popups::{
-    DeleteConfirmInputPopup, DeleteConfirmRadioPopup, ErrorPopup, FeedDeleteConfirmInputPopup,
-    FeedDeleteConfirmRadioPopup, HelpPopup, MessagePopup, PodcastAddPopup, QuitPopup,
-    SavePlaylistConfirm, SavePlaylistPopup,
+    ErrorPopup, HelpPopup, MessagePopup, PodcastAddPopup, QuitPopup, SavePlaylistConfirm,
+    SavePlaylistPopup,
 };
 pub use progress::Progress;
-pub use youtube_search::{YSInputPopup, YSTablePopup};
-//Tag Editor Controls,
 pub use tag_editor::*;
+use termusicplayback::SharedSettings;
+pub use youtube_search::{YSInputPopup, YSTablePopup};
 
 use crate::config::Keys;
-// #[cfg(any(feature = "mpris", feature = "discord"))]
-// use crate::track::Track;
 use crate::ui::{
     ConfigEditorMsg, GSMsg, Id, IdConfigEditor, IdTagEditor, Model, Msg, PLMsg, XYWHMsg, YSMsg,
 };
@@ -74,14 +72,14 @@ use tuirealm::{Component, Event, MockComponent, Sub, SubClause, SubEventClause};
 #[derive(MockComponent)]
 pub struct GlobalListener {
     component: Phantom,
-    keys: Keys,
+    config: SharedSettings,
 }
 
 impl GlobalListener {
-    pub fn new(keys: &Keys) -> Self {
+    pub fn new(config: SharedSettings) -> Self {
         Self {
             component: Phantom::default(),
-            keys: keys.clone(),
+            config,
         }
     }
 }
@@ -89,141 +87,124 @@ impl GlobalListener {
 impl Component<Msg, NoUserEvent> for GlobalListener {
     #[allow(clippy::too_many_lines)]
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+        let keys = &self.config.read().keys;
         match ev {
             Event::WindowResize(..) => Some(Msg::UpdatePhoto),
-            Event::Keyboard(keyevent) if keyevent == self.keys.global_esc.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.global_esc.key_event() => {
                 Some(Msg::QuitPopupShow)
             }
-            Event::Keyboard(keyevent) if keyevent == self.keys.global_quit.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.global_quit.key_event() => {
                 Some(Msg::QuitPopupShow)
             }
             Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_player_toggle_pause.key_event() =>
+                if keyevent == keys.global_player_toggle_pause.key_event() =>
             {
                 Some(Msg::PlayerTogglePause)
             }
-            Event::Keyboard(keyevent) if keyevent == self.keys.global_player_next.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.global_player_next.key_event() => {
                 Some(Msg::Playlist(PLMsg::NextSong))
             }
-            Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_player_previous.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.global_player_previous.key_event() => {
                 Some(Msg::Playlist(PLMsg::PrevSong))
             }
             Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_player_volume_minus_1.key_event() =>
+                if keyevent == keys.global_player_volume_minus_1.key_event() =>
             {
                 Some(Msg::PlayerVolumeDown)
             }
             Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_player_volume_minus_2.key_event() =>
+                if keyevent == keys.global_player_volume_minus_2.key_event() =>
             {
                 Some(Msg::PlayerVolumeDown)
             }
             Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_player_volume_plus_1.key_event() =>
+                if keyevent == keys.global_player_volume_plus_1.key_event() =>
             {
                 Some(Msg::PlayerVolumeUp)
             }
             Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_player_volume_plus_2.key_event() =>
+                if keyevent == keys.global_player_volume_plus_2.key_event() =>
             {
                 Some(Msg::PlayerVolumeUp)
             }
-            Event::Keyboard(keyevent) if keyevent == self.keys.global_help.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.global_help.key_event() => {
                 Some(Msg::HelpPopupShow)
             }
             Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_player_seek_forward.key_event() =>
+                if keyevent == keys.global_player_seek_forward.key_event() =>
             {
                 Some(Msg::PlayerSeekForward)
             }
             Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_player_seek_backward.key_event() =>
+                if keyevent == keys.global_player_seek_backward.key_event() =>
             {
                 Some(Msg::PlayerSeekBackward)
             }
-            Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_player_speed_up.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.global_player_speed_up.key_event() => {
                 Some(Msg::PlayerSpeedUp)
             }
-            Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_player_speed_down.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.global_player_speed_down.key_event() => {
                 Some(Msg::PlayerSpeedDown)
             }
 
             Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_lyric_adjust_forward.key_event() =>
+                if keyevent == keys.global_lyric_adjust_forward.key_event() =>
             {
                 Some(Msg::LyricAdjustDelay(1000))
             }
             Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_lyric_adjust_backward.key_event() =>
+                if keyevent == keys.global_lyric_adjust_backward.key_event() =>
             {
                 Some(Msg::LyricAdjustDelay(-1000))
             }
-            Event::Keyboard(keyevent) if keyevent == self.keys.global_lyric_cycle.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.global_lyric_cycle.key_event() => {
                 Some(Msg::LyricCycle)
             }
 
-            Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_layout_treeview.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.global_layout_treeview.key_event() => {
                 Some(Msg::LayoutTreeView)
             }
 
-            Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_layout_database.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.global_layout_database.key_event() => {
                 Some(Msg::LayoutDataBase)
             }
 
-            Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_layout_podcast.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.global_layout_podcast.key_event() => {
                 Some(Msg::LayoutPodCast)
             }
 
             Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_player_toggle_gapless.key_event() =>
+                if keyevent == keys.global_player_toggle_gapless.key_event() =>
             {
                 Some(Msg::PlayerToggleGapless)
             }
 
-            Event::Keyboard(keyevent) if keyevent == self.keys.global_config_open.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.global_config_open.key_event() => {
                 Some(Msg::ConfigEditor(ConfigEditorMsg::Open))
             }
 
-            Event::Keyboard(keyevent) if keyevent == self.keys.global_save_playlist.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.global_save_playlist.key_event() => {
                 Some(Msg::SavePlaylistPopupShow)
             }
-            Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_xywh_move_left.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_move_left.key_event() => {
                 Some(Msg::Xywh(XYWHMsg::MoveLeft))
             }
-            Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_xywh_move_right.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_move_right.key_event() => {
                 Some(Msg::Xywh(XYWHMsg::MoveRight))
             }
-            Event::Keyboard(keyevent) if keyevent == self.keys.global_xywh_move_up.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_move_up.key_event() => {
                 Some(Msg::Xywh(XYWHMsg::MoveUp))
             }
-            Event::Keyboard(keyevent)
-                if keyevent == self.keys.global_xywh_move_down.key_event() =>
-            {
+            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_move_down.key_event() => {
                 Some(Msg::Xywh(XYWHMsg::MoveDown))
             }
-            Event::Keyboard(keyevent) if keyevent == self.keys.global_xywh_zoom_in.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_zoom_in.key_event() => {
                 Some(Msg::Xywh(XYWHMsg::ZoomIn))
             }
-            Event::Keyboard(keyevent) if keyevent == self.keys.global_xywh_zoom_out.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_zoom_out.key_event() => {
                 Some(Msg::Xywh(XYWHMsg::ZoomOut))
             }
-            Event::Keyboard(keyevent) if keyevent == self.keys.global_xywh_hide.key_event() => {
+            Event::Keyboard(keyevent) if keyevent == keys.global_xywh_hide.key_event() => {
                 Some(Msg::Xywh(XYWHMsg::Hide))
             }
             _ => None,
