@@ -21,7 +21,6 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  */
-// use crate::config::Settings;
 use crate::ui::components::{
     LabelGeneric, LabelSpan, TECounterDelete, TEInputAlbum, TEInputArtist, TEInputGenre,
     TEInputTitle, TESelectLyric, TETableLyricOptions, TETextareaLyric,
@@ -40,8 +39,7 @@ use tuirealm::State;
 impl Model {
     #[allow(clippy::too_many_lines)]
     pub fn view_tag_editor(&mut self) {
-        assert!(self
-            .terminal
+        self.terminal
             .raw_mut()
             .draw(|f| {
                 let select_lyric_len =
@@ -170,13 +168,13 @@ impl Model {
                     }
                 }
             })
-            .is_ok());
+            .expect("Expected to draw without error");
     }
 
     pub fn mount_tageditor(&mut self, node_id: &str) {
         let p: &Path = Path::new(node_id);
         if p.is_dir() {
-            self.mount_error_popup("directory doesn't have tag!");
+            self.mount_error_popup(anyhow::anyhow!("{p:?} directory doesn't have tag!"));
             return;
         }
 
@@ -188,7 +186,10 @@ impl Model {
                     .app
                     .remount(
                         Id::TagEditor(IdTagEditor::LabelHint),
-                        Box::new(LabelGeneric::new(&self.config, "Press <ENTER> to search:")),
+                        Box::new(LabelGeneric::new(
+                            &self.config.read(),
+                            "Press <ENTER> to search:"
+                        )),
                         vec![]
                     )
                     .is_ok());
@@ -196,7 +197,7 @@ impl Model {
                     .app
                     .remount(
                         Id::TagEditor(IdTagEditor::InputArtist),
-                        Box::new(TEInputArtist::new(&self.config)),
+                        Box::new(TEInputArtist::new(self.config.clone())),
                         vec![]
                     )
                     .is_ok());
@@ -204,7 +205,7 @@ impl Model {
                     .app
                     .remount(
                         Id::TagEditor(IdTagEditor::InputTitle),
-                        Box::new(TEInputTitle::new(&self.config)),
+                        Box::new(TEInputTitle::new(self.config.clone())),
                         vec![]
                     )
                     .is_ok());
@@ -212,7 +213,7 @@ impl Model {
                     .app
                     .remount(
                         Id::TagEditor(IdTagEditor::InputAlbum),
-                        Box::new(TEInputAlbum::new(&self.config)),
+                        Box::new(TEInputAlbum::new(self.config.clone())),
                         vec![]
                     )
                     .is_ok());
@@ -220,7 +221,7 @@ impl Model {
                     .app
                     .remount(
                         Id::TagEditor(IdTagEditor::InputGenre),
-                        Box::new(TEInputGenre::new(&self.config)),
+                        Box::new(TEInputGenre::new(self.config.clone())),
                         vec![]
                     )
                     .is_ok());
@@ -228,7 +229,7 @@ impl Model {
                     .app
                     .remount(
                         Id::TagEditor(IdTagEditor::TableLyricOptions),
-                        Box::new(TETableLyricOptions::new(&self.config)),
+                        Box::new(TETableLyricOptions::new(self.config.clone())),
                         vec![]
                     )
                     .is_ok());
@@ -236,7 +237,7 @@ impl Model {
                     .app
                     .remount(
                         Id::TagEditor(IdTagEditor::SelectLyric),
-                        Box::new(TESelectLyric::new(&self.config)),
+                        Box::new(TESelectLyric::new(self.config.clone())),
                         vec![]
                     )
                     .is_ok());
@@ -244,7 +245,7 @@ impl Model {
                     .app
                     .remount(
                         Id::TagEditor(IdTagEditor::CounterDelete),
-                        Box::new(TECounterDelete::new(5, &self.config)),
+                        Box::new(TECounterDelete::new(5, self.config.clone())),
                         vec![]
                     )
                     .is_ok());
@@ -252,7 +253,7 @@ impl Model {
                     .app
                     .remount(
                         Id::TagEditor(IdTagEditor::TextareaLyric),
-                        Box::new(TETextareaLyric::new(&self.config)),
+                        Box::new(TETextareaLyric::new(self.config.clone())),
                         vec![]
                     )
                     .is_ok());
@@ -263,11 +264,11 @@ impl Model {
                 self.init_by_song(&s);
             }
             Err(e) => {
-                self.mount_error_popup(format!("song load error: {e}"));
+                self.mount_error_popup(e.context("track parse"));
             }
         };
         if let Err(e) = self.update_photo() {
-            self.mount_error_popup(format!("clear photo error: {e}"));
+            self.mount_error_popup(e.context("update_photo"));
         }
     }
     pub fn umount_tageditor(&mut self) {
@@ -299,7 +300,7 @@ impl Model {
             .umount(&Id::TagEditor(IdTagEditor::TextareaLyric))
             .ok();
         if let Err(e) = self.update_photo() {
-            self.mount_error_popup(format!("update photo error: {e}"));
+            self.mount_error_popup(e.context("update_photo"));
         }
     }
 
@@ -461,66 +462,57 @@ impl Model {
     }
 
     pub fn remount_tag_editor_label_help(&mut self) {
+        let config = self.config.read();
         assert!(self
             .app
             .remount(
                 Id::Label,
                 Box::new(LabelSpan::new(
-                    &self.config,
+                    &config,
                     &[
-                        TextSpan::new(format!("<{}>", self.config.keys.config_save))
+                        TextSpan::new(format!("<{}>", config.keys.config_save))
                             .bold()
-                            .fg(self
-                                .config
+                            .fg(config
                                 .style_color_symbol
                                 .library_highlight()
                                 .unwrap_or(Color::Cyan)),
-                        TextSpan::new(" Save tag ").fg(self
-                            .config
+                        TextSpan::new(" Save tag ").fg(config
                             .style_color_symbol
                             .library_foreground()
                             .unwrap_or(Color::White)),
-                        TextSpan::new(format!("<{}>", self.config.keys.global_esc))
+                        TextSpan::new(format!("<{}>", config.keys.global_esc))
                             .bold()
-                            .fg(self
-                                .config
+                            .fg(config
                                 .style_color_symbol
                                 .library_highlight()
                                 .unwrap_or(Color::Cyan)),
-                        TextSpan::new(" Exit ").fg(self
-                            .config
+                        TextSpan::new(" Exit ").fg(config
                             .style_color_symbol
                             .library_foreground()
                             .unwrap_or(Color::White)),
-                        TextSpan::new("<Tab/ShiftTab>").bold().fg(self
-                            .config
+                        TextSpan::new("<Tab/ShiftTab>").bold().fg(config
                             .style_color_symbol
                             .library_highlight()
                             .unwrap_or(Color::Cyan)),
-                        TextSpan::new(" Change field ").fg(self
-                            .config
+                        TextSpan::new(" Change field ").fg(config
                             .style_color_symbol
                             .library_foreground()
                             .unwrap_or(Color::White)),
-                        TextSpan::new("<ENTER>").bold().fg(self
-                            .config
+                        TextSpan::new("<ENTER>").bold().fg(config
                             .style_color_symbol
                             .library_highlight()
                             .unwrap_or(Color::Cyan)),
-                        TextSpan::new(" Search/Embed tag ").fg(self
-                            .config
+                        TextSpan::new(" Search/Embed tag ").fg(config
                             .style_color_symbol
                             .library_foreground()
                             .unwrap_or(Color::White)),
-                        TextSpan::new(format!("<{}>", self.config.keys.library_search_youtube))
+                        TextSpan::new(format!("<{}>", config.keys.library_search_youtube))
                             .bold()
-                            .fg(self
-                                .config
+                            .fg(config
                                 .style_color_symbol
                                 .library_highlight()
                                 .unwrap_or(Color::Cyan)),
-                        TextSpan::new(" download ").fg(self
-                            .config
+                        TextSpan::new(" download ").fg(config
                             .style_color_symbol
                             .library_foreground()
                             .unwrap_or(Color::White)),
